@@ -402,11 +402,13 @@ crs.formula <- function(formula,
                         model.return=FALSE,
                         restarts=0,
                         random.seed=42,
-                        opts=list("MAX_BB_EVAL"=10000,
-                                  "EPSILON"=.Machine$double.eps,
-                                  "INITIAL_MESH_SIZE"="r1.0e-01",
-                                  "MIN_MESH_SIZE"=paste("r",sqrt(.Machine$double.eps),sep=""),
-                                  "MIN_POLL_SIZE"=paste("r",sqrt(.Machine$double.eps),sep="")),
+                        max.bb.eval=10000,
+                        initial.mesh.size.real="r1.0e-01", 
+                        initial.mesh.size.integer="1",
+                        min.mesh.size.real=paste(sqrt(.Machine$double.eps)),
+                        min.mesh.size.integer=paste(sqrt(.Machine$double.eps)),
+                        min.poll.size.real=paste(sqrt(.Machine$double.eps)),
+                        min.poll.size.integer=paste(sqrt(.Machine$double.eps)),
                         nmulti=5,
                         tau=NULL,
                         weights=NULL,
@@ -555,7 +557,10 @@ crs.formula <- function(formula,
                                                        segments=segments,
                                                        include=include,
                                                        random.seed=random.seed,
-                                                       opts=opts,
+                                                       max.bb.eval=max.bb.eval,
+                                                       initial.mesh.size.integer=initial.mesh.size.integer,
+                                                       min.mesh.size.integer=min.mesh.size.integer,
+                                                       min.poll.size.integer=min.poll.size.integer,
                                                        nmulti=nmulti,
                                                        tau=tau,
                                                        weights=weights,
@@ -618,7 +623,13 @@ crs.formula <- function(formula,
                                                        lambda.discrete=lambda.discrete,
                                                        lambda.discrete.num=lambda.discrete.num,
                                                        random.seed=random.seed,
-                                                       opts=opts,
+                                                       max.bb.eval=max.bb.eval,
+                                                       initial.mesh.size.real=initial.mesh.size.real,
+                                                       initial.mesh.size.integer=initial.mesh.size.integer,
+                                                       min.mesh.size.real=min.mesh.size.real,
+                                                       min.mesh.size.integer=min.mesh.size.integer,
+                                                       min.poll.size.real=min.poll.size.real,
+                                                       min.poll.size.integer=min.poll.size.integer,
                                                        nmulti=nmulti,
                                                        tau=tau,
                                                        weights=weights,
