@@ -1,5 +1,5 @@
 /*-------------------------------------------------------------------------------------*/
-/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.5.1        */
+/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.6.2        */
 /*                                                                                     */
 /*  Copyright (C) 2001-2012  Mark Abramson        - the Boeing Company, Seattle        */
 /*                           Charles Audet        - Ecole Polytechnique, Montreal      */
@@ -40,8 +40,9 @@
   \date   2010-03-23
   \see    utils.hpp
 */
-#include "utils.hpp"
-using namespace std;  //zhenghua
+# include "utils.hpp"
+using namespace std; // zhenghua
+
 /*---------------------------------------------------------------*/
 /*               construct the first n prime numbers             */
 /*---------------------------------------------------------------*/
@@ -149,6 +150,17 @@ std::string NOMAD::itos ( int i )
   return oss.str();
 }
 
+/*-----------------------------------------------------------------*/
+/*                         NOMAD::itos                             */
+/*-----------------------------------------------------------------*/
+std::string NOMAD::itos ( size_t i )
+{
+    std::ostringstream oss;
+    oss << i;
+    return oss.str();
+}
+
+
 /*-----------------------------------------------------------------*/
 /*                         NOMAD::toupper - 1/2                    */
 /*-----------------------------------------------------------------*/
@@ -223,14 +235,16 @@ bool NOMAD::bbot_is_constraint ( NOMAD::bb_output_type bbot )
 /*-----------------------------------------------------------------------*/
 bool NOMAD::dir_is_mads ( NOMAD::direction_type dt )
 {
-  return ( dt == NOMAD::ORTHO_1   ||
-	   dt == NOMAD::ORTHO_2   ||
-	   dt == NOMAD::ORTHO_NP1 ||
-	   dt == NOMAD::ORTHO_2N  ||
-	   dt == NOMAD::LT_1      ||
-	   dt == NOMAD::LT_2      ||
-	   dt == NOMAD::LT_2N     ||
-	   dt == NOMAD::LT_NP1      );
+  return (dt == NOMAD::ORTHO_1        ||
+		  dt == NOMAD::ORTHO_2        ||
+		  dt == NOMAD::ORTHO_NP1_QUAD ||
+		  dt == NOMAD::ORTHO_NP1_NEG  ||
+		  dt == NOMAD::DYN_ADDED      ||
+		  dt == NOMAD::ORTHO_2N       ||
+		  dt == NOMAD::LT_1           ||
+		  dt == NOMAD::LT_2           ||
+		  dt == NOMAD::LT_2N          ||
+		  dt == NOMAD::LT_NP1      );
 }
 
 /*----------------------------------------------------------------------*/
@@ -277,17 +291,18 @@ bool NOMAD::dir_is_random ( NOMAD::direction_type dt )
 /*-----------------------------------------------------------------------------*/
 bool NOMAD::dir_is_orthomads ( NOMAD::direction_type dt )
 {
-  return ( dt == NOMAD::ORTHO_1   ||
-	   dt == NOMAD::ORTHO_2   ||
-	   dt == NOMAD::ORTHO_NP1 ||
-	   dt == NOMAD::ORTHO_2N     );
+  return (dt == NOMAD::ORTHO_1        ||
+		  dt == NOMAD::ORTHO_2        ||
+		  dt == NOMAD::ORTHO_NP1_QUAD ||
+		  dt == NOMAD::ORTHO_NP1_NEG  ||
+		  dt == NOMAD::ORTHO_2N         );
 }
 
 /*---------------------------------------------------------------------*/
-/*  check if a set of directions include Ortho-MADS direction          */
+/*  check if a set of directions includes one Ortho-MADS direction     */
 /*  (true if at least one direction in the set is of type Ortho-MADS)  */
 /*---------------------------------------------------------------------*/
-bool NOMAD::dirs_are_orthomads ( const std::set<NOMAD::direction_type> & dir_types )
+bool NOMAD::dirs_have_orthomads ( const std::set<NOMAD::direction_type> & dir_types )
 {
   std::set<NOMAD::direction_type>::const_iterator it , end = dir_types.end();
   for ( it = dir_types.begin() ; it != end ; ++it )
@@ -296,6 +311,19 @@ bool NOMAD::dirs_are_orthomads ( const std::set<NOMAD::direction_type> & dir_typ
   return false;
 }
 
+/*---------------------------------------------------------------------*/
+/*  check if a set of directions includes one Ortho-MADS N+1 direction */
+/*  (true if at least one direction in the set is of type Ortho-MADS)  */
+/*---------------------------------------------------------------------*/
+bool NOMAD::dirs_have_orthomads_np1( const std::set<NOMAD::direction_type> & dir_types )
+{
+	std::set<NOMAD::direction_type>::const_iterator it , end = dir_types.end();
+	for ( it = dir_types.begin() ; it != end ; ++it )
+		if ( (*it)==NOMAD::ORTHO_NP1_QUAD || (*it)==NOMAD::ORTHO_NP1_NEG)
+			return true;
+	return false;
+}
+
 /*---------------------------------------------------*/
 /*  returns true if one of the string of ls is in s  */
 /*---------------------------------------------------*/
@@ -316,166 +344,203 @@ bool NOMAD::string_find ( const std::string & s1 , const std::string & s2 )
   return ( s1.find(s2) < s1.size() );
 }
 
+/*--------------------------------------------------------------------------*/
+/*        returns true if one of the string in ls is an element  of s       */
+/*--------------------------------------------------------------------------*/
+bool NOMAD::string_match ( const std::string & s , const std::list<std::string> & ls )
+{
+	std::list<std::string>::const_iterator it , end = ls.end();
+	for ( it = ls.begin() ; it != end ; ++it )
+		if ( s.compare(*it) == 0 )
+			return true;
+	return false;
+}
+
+
 /*-----------------------------------------------------------------*/
 /*         interpret a list of strings as a direction type         */
 /*-----------------------------------------------------------------*/
 bool NOMAD::strings_to_direction_type ( const std::list<std::string> & ls ,
 					NOMAD::direction_type        & dt   )
 {
-  
-  dt = NOMAD::UNDEFINED_DIRECTION;
-
-  if ( ls.empty() || ls.size() > 4 )
-    return false;
-
-  std::list<std::string>::const_iterator it = ls.begin() , end = ls.end();
-  std::string                            s  = *it;
-  NOMAD::toupper ( s );
-
-  // no direction:
-  if ( s == "NONE" ) {
-    dt = NOMAD::NO_DIRECTION;
-    return true;
-  }
-
-  // Ortho-MADS with 1, 2, n+1, or 2n directions:
-  if ( s == "ORTHO" ) {
-    ++it;
-    if ( it == end ) {
-      dt = NOMAD::ORTHO_2N;
-      return true;
-    }
-    if ( *it == "1" ) {
-      dt = NOMAD::ORTHO_1;
-      return true;
-    }
-    if ( *it == "2" ) {
-      dt = NOMAD::ORTHO_2;
-      return true;
-    }
-    s = *it;
-    NOMAD::toupper ( s );
-    if ( s == "N+1" ) {
-      dt = NOMAD::ORTHO_NP1;
-      return true;
-    }
-    if ( s == "2N" ) {
-      dt = NOMAD::ORTHO_2N;
-      return true;
-    }
-    return false;
-  }
-
-  // LT-MADS with 1, 2 or 2n directions:
-  if ( s == "LT" ) {
-    ++it;
-    if ( it == end ) {
-      dt = NOMAD::LT_2N;
-      return true;
-    }
-    if ( *it == "1" ) {
-      dt = NOMAD::LT_1;
-      return true;
-    }
-    if ( *it == "2" ) {
-      dt = NOMAD::LT_2;
-      return true;
-    }
-    s = *it;
-    NOMAD::toupper ( s );
-    if ( s == "N+1" ) {
-      dt = NOMAD::LT_NP1;
-      return true;
-    }
-    if ( s == "2N" ) {
-      dt = NOMAD::LT_2N;
-      return true;
-    }
-    return false;
-  }
-
-  // GPS:
-  if ( s == "GPS" ) {
-    ++it;
-    if ( it == end ) {
-      dt = NOMAD::GPS_2N_STATIC;
-      return true;
-    }
-    s = *it;
-    NOMAD::toupper ( s );
-
-    // GPS for binary variables:
-    if ( s == "BINARY" || s == "BIN" ) {
-      dt = NOMAD::GPS_BINARY;
-      return true;
-    }
-
-    // GPS, n+1 directions:
-    if ( s == "N+1" ) {
-      ++it;
-      if ( it == end ) {
-	dt = NOMAD::GPS_NP1_STATIC;
-	return true;
-      }
-      s = *it;
-      NOMAD::toupper ( s );
-
-      // GPS, n+1, static:
-      if ( s == "STATIC" ) {
-	++it;
-	if ( it == end ) {
-	  dt = NOMAD::GPS_NP1_STATIC;
-	  return true;
-	}
-	s = *it;
+	
+	dt = NOMAD::UNDEFINED_DIRECTION;
+	
+	if ( ls.empty() || ls.size() > 4 )
+		return false;
+	
+	std::list<std::string>::const_iterator it = ls.begin() , end = ls.end();
+	std::string                            s  = *it;
 	NOMAD::toupper ( s );
-	if ( s == "UNIFORM" ) {
-	  dt = NOMAD::GPS_NP1_STATIC_UNIFORM;
-	  return true;
+	
+	// no direction:
+	if ( s == "NONE" ) {
+		dt = NOMAD::NO_DIRECTION;
+		return true;
+	}
+	
+	// Ortho-MADS with 1, 2, n+1 (plus QUAD or NEG), or 2n directions:
+	if ( s == "ORTHO" ) 
+	{
+		++it;
+		if ( it == end )
+		{
+			dt = NOMAD::ORTHO_NP1_QUAD;  // Default for ORTHO
+			return true;
+		}
+		if ( *it == "1" )
+		{
+			dt = NOMAD::ORTHO_1;
+			return true;
+		}
+		if ( *it == "2" )
+		{
+			dt = NOMAD::ORTHO_2;
+			return true;
+		}
+		s = *it;
+		NOMAD::toupper ( s );
+		if ( s == "2N" )
+		{
+			dt = NOMAD::ORTHO_2N;
+			return true;
+		}
+		if ( s == "N+1" )
+		{
+			++it;
+			if (it==end)
+			{
+				dt = NOMAD::ORTHO_NP1_QUAD;   // Default for ORTHO N+1
+				return true;
+			}
+			s = *it;
+			NOMAD::toupper ( s );
+			if ( s=="QUAD" )
+			{
+				dt= NOMAD::ORTHO_NP1_QUAD;
+				return true;
+			}
+			if ( s=="NEG" )
+			{
+				dt=NOMAD::ORTHO_NP1_NEG;
+				return true;
+			}
+			
+		}
+		
+		return false;
+	}
+	
+	// LT-MADS with 1, 2 or 2n directions:
+	if ( s == "LT" ) {
+		++it;
+		if ( it == end ) {
+			dt = NOMAD::LT_2N;
+			return true;
+		}
+		if ( *it == "1" ) {
+			dt = NOMAD::LT_1;
+			return true;
+		}
+		if ( *it == "2" ) {
+			dt = NOMAD::LT_2;
+			return true;
+		}
+		s = *it;
+		NOMAD::toupper ( s );
+		if ( s == "N+1" ) {
+			dt = NOMAD::LT_NP1;
+			return true;
+		}
+		if ( s == "2N" ) {
+			dt = NOMAD::LT_2N;
+			return true;
+		}
+		return false;
+	}
+	
+	// GPS:
+	if ( s == "GPS" ) {
+		++it;
+		if ( it == end ) {
+			dt = NOMAD::GPS_2N_STATIC;
+			return true;
+		}
+		s = *it;
+		NOMAD::toupper ( s );
+		
+		// GPS for binary variables:
+		if ( s == "BINARY" || s == "BIN" ) {
+			dt = NOMAD::GPS_BINARY;
+			return true;
+		}
+		
+		// GPS, n+1 directions:
+		if ( s == "N+1" ) {
+			++it;
+			if ( it == end ) {
+				dt = NOMAD::GPS_NP1_STATIC;
+				return true;
+			}
+			s = *it;
+			NOMAD::toupper ( s );
+			
+			// GPS, n+1, static:
+			if ( s == "STATIC" ) {
+				++it;
+				if ( it == end ) {
+					dt = NOMAD::GPS_NP1_STATIC;
+					return true;
+				}
+				s = *it;
+				NOMAD::toupper ( s );
+				if ( s == "UNIFORM" ) {
+					dt = NOMAD::GPS_NP1_STATIC_UNIFORM;
+					return true;
+				}
+				return false;
+			}
+			
+			// GPS, n+1, random:
+			if ( s == "RAND" || s == "RANDOM" ) {
+				++it;
+				if ( it == end ) {
+					dt = NOMAD::GPS_NP1_RAND;
+					return true;
+				}
+				s = *it;
+				NOMAD::toupper ( s );
+				if ( s == "UNIFORM" ) {
+					dt = NOMAD::GPS_NP1_RAND_UNIFORM;
+					return true;
+				}
+				return false;
+			}
+			return false;
+		}
+		
+		// 2n directions:
+		if ( s == "2N" ) {
+			++it;
+			if ( it == end ) {
+				dt = NOMAD::GPS_2N_STATIC;
+				return true;
+			}
+			s = *it;
+			NOMAD::toupper ( s );
+			if ( s == "STATIC" ) {
+				dt = NOMAD::GPS_2N_STATIC;
+				return true;
+			}
+			if ( s == "RAND" || s == "RANDOM" ) {
+				dt = NOMAD::GPS_2N_RAND;
+				return true;
+			}
+			return false;
+		}
+		return false;
 	}
 	return false;
-      }
-
-      // GPS, n+1, random:
-      if ( s == "RAND" || s == "RANDOM" ) {
-	++it;
-	if ( it == end ) {
-	  dt = NOMAD::GPS_NP1_RAND;
-	  return true;
-	}
-	s = *it;
-	NOMAD::toupper ( s );
-	if ( s == "UNIFORM" ) {
-	  dt = NOMAD::GPS_NP1_RAND_UNIFORM;
-	  return true;
-	}
-	return false;
-      }
-      return false;
-    }
-    
-    // 2n directions:
-    if ( s == "2N" ) {
-      ++it;
-      if ( it == end ) {
-	dt = NOMAD::GPS_2N_STATIC;
-	return true;
-      }
-      s = *it;
-      NOMAD::toupper ( s );
-      if ( s == "STATIC" ) {
-	dt = NOMAD::GPS_2N_STATIC;
-	return true;
-      }
-      if ( s == "RAND" || s == "RANDOM" ) {
-	dt = NOMAD::GPS_2N_RAND;
-	return true;
-      }
-      return false;
-    }
-    return false;
-  }
-  return false;
 }
 
 /*---------------------------------------*/
@@ -746,6 +811,45 @@ bool NOMAD::string_to_index_range ( const std::string & s           ,
   return !check_order || i <= j;
 }
 
+
+int NOMAD::get_rank(double ** M,
+					size_t m,
+					size_t n)
+{
+	double  * W = new double  [n];
+	double ** V = new double *[n];
+	for (size_t i = 0 ; i < n ; ++i )
+	{
+		V[i]=new double [n];
+	}
+	
+	std::string error_msg;
+	NOMAD::SVD_decomposition ( error_msg , M , W , V , static_cast<int>(m) , static_cast<int>(n) );
+
+	for (size_t i=0;i<n;++i)
+		delete [] V[i];
+	delete [] V;
+
+	
+ 	if (! error_msg.empty())
+	{
+		delete [] W;
+		return -1;
+	}
+	
+	int rank=0;
+	for (size_t i=0;i<n;i++)
+	{
+		if (fabs(W[i])>NOMAD::SVD_EPS)
+			rank++;
+	}
+	
+	delete [] W;
+	return rank;
+	
+}
+
+
 /*--------------------------------------------------------------*/
 /*                        SVD decomposition                     */
 /*  inspired and recoded from an old numerical recipes version  */
@@ -770,13 +874,6 @@ bool NOMAD::string_to_index_range ( const std::string & s           ,
 /*           V is given, not V'                                 */
 /*                                                              */
 /*--------------------------------------------------------------*/
-/* 2011-08-16 -- BUG REPORT (found by Etienne Duclos)           */
-/*                                                              */
-/* the -Wall option gave a warning when nm was not initialized  */
-/*                                                              */
-/* Solution: initialize nm = 0                                  */
-/*                                                              */
-/*--------------------------------------------------------------*/
 bool NOMAD::SVD_decomposition ( std::string & error_msg ,
 				double     ** M         ,
 				double      * W         ,
@@ -785,256 +882,285 @@ bool NOMAD::SVD_decomposition ( std::string & error_msg ,
 				int           n         ,
 				int           max_mpn     ) // default=1500
 {
-  error_msg.clear();
-
-  if ( max_mpn > 0 && m+n > max_mpn ) {
-    error_msg = "SVD_decomposition() error: m+n > " + NOMAD::itos ( max_mpn );
-    return false;
-  }
-
-  double * rv1   = new double[n];
-  double   scale = 0.0;
-  double   g     = 0.0;
-  double   norm  = 0.0;
-
-  int      nm1   = n - 1;
-
-  bool   flag;
-  int    i , j , k , l , its , jj , nm = 0;
-  double s , f , h , tmp , c , x , y , z , absf , absg , absh;
-
-  const int NITER = 30;
-
-  // Householder reduction to bidiagonal form:
-  for ( i = 0 ; i < n ; ++i ) {
-    l      = i + 1;
-    rv1[i] = scale * g;
-    g      = s = scale = 0.0;
-    if ( i < m ) {
-      for ( k = i ; k < m ; ++k )
- 	scale += fabs ( M[k][i] );
-      if ( scale ) {
- 	for ( k = i ; k < m ; ++k ) {
- 	  M[k][i] /= scale;
- 	  s += M[k][i] * M[k][i];
+	error_msg.clear();
+	
+	if ( max_mpn > 0 && m+n > max_mpn ) {
+		error_msg = "SVD_decomposition() error: m+n > " + NOMAD::itos ( max_mpn );
+		return false;
 	}
- 	f       = M[i][i];
-	g       = ( f >= 0.0 ) ? -fabs(sqrt(s)) : fabs(sqrt(s));
- 	h       = f * g - s;
-	M[i][i] = f - g;
- 	for ( j = l ; j < n ; ++j ) {
- 	  for ( s = 0.0 , k = i ; k < m ; ++k )
- 	    s += M[k][i] * M[k][j];
- 	  f = s / h;
- 	  for ( k = i ; k < m ; ++k )
- 	    M[k][j] += f * M[k][i];
- 	}
- 	for ( k = i ; k < m ; ++k )
- 	  M[k][i] *= scale;
-      }
-    }
-    W[i] = scale * g;
-    g    = s = scale = 0.0;
-    if ( i < m && i != nm1 ) {
-      for ( k = l ; k < n ; ++k )
- 	scale += fabs ( M[i][k] );
-      if ( scale ) {
- 	for ( k = l ; k < n ; ++k ) {
- 	  M[i][k] /= scale;
- 	  s       += M[i][k] * M[i][k];
- 	}
- 	f       = M[i][l];
-	g       = ( f >= 0.0 ) ? -fabs(sqrt(s)) : fabs(sqrt(s));
- 	h       = f * g - s;
- 	M[i][l] = f - g;
- 	for ( k = l ; k < n ; ++k )
- 	  rv1[k] = M[i][k] / h;
- 	for ( j = l ; j < m ; ++j ) {
- 	  for ( s=0.0,k=l ; k < n ; ++k )
- 	    s += M[j][k] * M[i][k];
- 	  for ( k=l ; k < n ; ++k )
- 	    M[j][k] += s * rv1[k];
- 	}
- 	for ( k = l ; k < n ; ++k )
- 	  M[i][k] *= scale;
-      }
-    }
-    tmp  = fabs ( W[i] ) + fabs ( rv1[i] );
-    norm = ( norm > tmp ) ? norm : tmp;
-  }
-
-  // accumulation of right-hand transformations:
-  for ( i = nm1 ; i >= 0 ; --i ) {
-    if ( i < nm1 ) {
-      if ( g ) {
- 	for ( j = l ; j < n ; ++j )
- 	  V[j][i] = ( M[i][j] / M[i][l] ) / g;
- 	for ( j = l ; j < n ; ++j ) {
- 	  for ( s = 0.0 , k = l ; k < n ; ++k )
- 	    s += M[i][k] * V[k][j];
- 	  for ( k = l ; k < n ; ++k )
- 	    V[k][j] += s * V[k][i];
- 	}
-      }
-      for ( j = l ; j < n ; ++j )
- 	V[i][j] = V[j][i] = 0.0;
-    }
-    V[i][i] = 1.0;
-    g       = rv1[i];
-    l       = i;
-  }
-
-  // accumulation of left-hand transformations:
-  for ( i = ( ( m < n ) ? m : n ) - 1 ; i >= 0 ; --i ) {
-    l = i + 1;
-    g = W[i];
-    for ( j = l ; j < n ; ++j )
-      M[i][j] = 0.0;
-    if ( g ) {
-      g = 1.0 / g;
-      for ( j = l ; j < n ; ++j ) {
-	for ( s = 0.0 , k = l ; k < m ; ++k )
-	  s += M[k][i] * M[k][j];
-	f = ( s / M[i][i] ) * g;
-	for ( k = i ; k < m ; ++k )
-	  M[k][j] += f * M[k][i];
-      }
-      for ( j = i ; j < m ; ++j )
-	M[j][i] *= g;
-    }
-    else
-      for ( j = i ; j < m ; ++j )
-	M[j][i] = 0.0;
-    ++M[i][i];
-  }
-
-  // diagonalization of the bidiagonal form:
-  for ( k = nm1 ; k >= 0 ; --k ) {
-    for ( its = 1 ; its <= NITER ; its++ ) {
-      flag = true;
-      for ( l = k ; l >= 0 ; l-- ) {
- 	nm = l - 1;
- 	if ( nm < 0 || fabs ( rv1[l]) + norm == norm ) {
- 	  flag = false;
- 	  break;
+	
+	double * rv1   = new double[n];
+	double   scale = 0.0;
+	double   g     = 0.0;
+	double   norm  = 0.0;
+	
+	int      nm1   = n - 1;
+	
+	bool   flag;
+	int    i , j , k , l , its , jj , nm = 0;
+	double s , f , h , tmp , c , x , y , z , absf , absg , absh;
+	
+	const int NITER = 30;
+	
+	// Initialization W and V
+	for (i=0; i < n; ++i)
+	{
+		W[i]=0.0;
+		for (j=0; j < n ; ++j)
+			V[i][j]=0.0;
 	}
- 	if ( fabs ( W[nm] ) + norm == norm )
-	  break;
-      }
-      if ( flag ) {
- 	c = 0.0;
- 	s = 1.0;
- 	for ( i = l ; i <= k ; i++ ) {
- 	  f      = s * rv1[i];
- 	  rv1[i] = c * rv1[i];
- 	  if ( fabs(f) + norm == norm )
- 	    break;
- 	  g = W[i];
-
-	  absf = fabs(f);
-	  absg = fabs(g);
-	  h    = ( absf > absg ) ?
-	    absf * sqrt ( 1.0 + pow ( absg/absf , 2.0 ) ) :
-	    ( ( absg==0 ) ? 0.0 : absg * sqrt ( 1.0 + pow ( absf/absg , 2.0 ) ) );
-
- 	  W[i] =  h;
- 	  h    =  1.0 / h;
- 	  c    =  g * h;
- 	  s    = -f * h;
- 	  for ( j = 0 ; j < m ; ++j ) {
- 	    y = M[j][nm];
- 	    z = M[j][ i];
- 	    M[j][nm] = y * c + z * s;
- 	    M[j][ i] = z * c - y * s;
- 	  }
- 	}
-      }
-      z = W[k];
-      if ( l == k) {
-	if ( z < 0.0 ) {
- 	  W[k] = -z;
- 	  for ( j = 0 ; j < n ; j++ )
- 	    V[j][k] = -V[j][k];
- 	}
- 	break;  // this 'break' is always active if k==0
-      }
-      if ( its == NITER ) {
-	error_msg = "SVD_decomposition() error: no convergence in " +
+	
+	// Householder reduction to bidiagonal form:
+	for ( i = 0 ; i < n ; ++i )
+	{
+		l      = i + 1;
+		rv1[i] = scale * g;
+		g      = s = scale = 0.0;
+		if ( i < m )
+		{
+			for ( k = i ; k < m ; ++k )
+				scale += fabs ( M[k][i] );
+			if ( scale ) {
+				for ( k = i ; k < m ; ++k ) 
+				{
+					M[k][i] /= scale;
+					s += M[k][i] * M[k][i];
+				}
+				f       = M[i][i];
+				g       = ( f >= 0.0 ) ? -fabs(sqrt(s)) : fabs(sqrt(s));
+				h       = f * g - s;
+				M[i][i] = f - g;
+				for ( j = l ; j < n ; ++j ) {
+					for ( s = 0.0 , k = i ; k < m ; ++k )
+						s += M[k][i] * M[k][j];
+					f = s / h;
+					for ( k = i ; k < m ; ++k )
+						M[k][j] += f * M[k][i];
+				}
+				for ( k = i ; k < m ; ++k )
+					M[k][i] *= scale;
+			}
+		}
+		W[i] = scale * g;
+		g    = s = scale = 0.0;
+		if ( i < m && i != nm1 ) 
+		{
+			for ( k = l ; k < n ; ++k )
+				scale += fabs ( M[i][k] );
+			if ( scale ) 
+			{
+				for ( k = l ; k < n ; ++k )
+				{
+					M[i][k] /= scale;
+					s       += M[i][k] * M[i][k];
+				}
+				f       = M[i][l];
+				g       = ( f >= 0.0 ) ? -fabs(sqrt(s)) : fabs(sqrt(s));
+				h       = f * g - s;
+				M[i][l] = f - g;
+				for ( k = l ; k < n ; ++k )
+					rv1[k] = M[i][k] / h;
+				for ( j = l ; j < m ; ++j ) {
+					for ( s=0.0,k=l ; k < n ; ++k )
+						s += M[j][k] * M[i][k];
+					for ( k=l ; k < n ; ++k )
+						M[j][k] += s * rv1[k];
+				}
+				for ( k = l ; k < n ; ++k )
+					M[i][k] *= scale;
+			}
+		}
+		tmp  = fabs ( W[i] ) + fabs ( rv1[i] );
+		norm = ( norm > tmp ) ? norm : tmp;
+	}
+	
+	// accumulation of right-hand transformations:
+	for ( i = nm1 ; i >= 0 ; --i )
+	{
+		if ( i < nm1 ) 
+		{
+			if ( g ) 
+			{
+				for ( j = l ; j < n ; ++j )
+					V[j][i] = ( M[i][j] / M[i][l] ) / g;
+				for ( j = l ; j < n ; ++j ) {
+					for ( s = 0.0 , k = l ; k < n ; ++k )
+						s += M[i][k] * V[k][j];
+					for ( k = l ; k < n ; ++k )
+						V[k][j] += s * V[k][i];
+				}
+			}
+			for ( j = l ; j < n ; ++j )
+				V[i][j] = V[j][i] = 0.0;
+		}
+		V[i][i] = 1.0;
+		g       = rv1[i];
+		l       = i;
+	}
+	
+	// accumulation of left-hand transformations:
+	for ( i = ( ( m < n ) ? m : n ) - 1 ; i >= 0 ; --i ) 
+	{
+		l = i + 1;
+		g = W[i];
+		for ( j = l ; j < n ; ++j )
+			M[i][j] = 0.0;
+		if ( g ) {
+			g = 1.0 / g;
+			for ( j = l ; j < n ; ++j ) 
+			{
+				for ( s = 0.0 , k = l ; k < m ; ++k )
+					s += M[k][i] * M[k][j];
+				f = ( s / M[i][i] ) * g;
+				for ( k = i ; k < m ; ++k )
+					M[k][j] += f * M[k][i];
+			}
+			for ( j = i ; j < m ; ++j )
+				M[j][i] *= g;
+		}
+		else
+			for ( j = i ; j < m ; ++j )
+				M[j][i] = 0.0;
+		++M[i][i];
+	}
+	
+	// diagonalization of the bidiagonal form:
+	for ( k = nm1 ; k >= 0 ; --k )
+	{
+		for ( its = 1 ; its <= NITER ; its++ ) 
+		{
+			flag = true;
+			for ( l = k ; l >= 0 ; l-- ) 
+			{
+				nm = l - 1;
+				if ( nm < 0 || fabs ( rv1[l]) + norm == norm )
+				{
+					flag = false;
+					break;
+				}
+				if ( fabs ( W[nm] ) + norm == norm )
+					break;
+			}
+			if ( flag )
+			{
+				c = 0.0;
+				s = 1.0;
+				for ( i = l ; i <= k ; i++ )
+				{
+					f      = s * rv1[i];
+					rv1[i] = c * rv1[i];
+					if ( fabs(f) + norm == norm )
+						break;
+					g = W[i];
+					
+					absf = fabs(f);
+					absg = fabs(g);
+					h    = ( absf > absg ) ?
+					absf * sqrt ( 1.0 + pow ( absg/absf , 2.0 ) ) :
+					( ( absg==0 ) ? 0.0 : absg * sqrt ( 1.0 + pow ( absf/absg , 2.0 ) ) );
+					
+					W[i] =  h;
+					h    =  1.0 / h;
+					c    =  g * h;
+					s    = -f * h;
+					for ( j = 0 ; j < m ; ++j ) {
+						y = M[j][nm];
+						z = M[j][ i];
+						M[j][nm] = y * c + z * s;
+						M[j][ i] = z * c - y * s;
+					}
+				}
+			}
+			z = W[k];
+			if ( l == k) {
+				if ( z < 0.0 ) {
+					W[k] = -z;
+					for ( j = 0 ; j < n ; j++ )
+						V[j][k] = -V[j][k];
+				}
+				break;  // this 'break' is always active if k==0
+			}
+			if ( its == NITER ) 
+			{
+				error_msg = "SVD_decomposition() error: no convergence in " +
 	            NOMAD::itos ( NITER ) + " iterations";
+				delete [] rv1;
+				return false;
+			}
+			x  = W[l];
+			nm = k - 1;
+			y  = W[nm];
+			g  = rv1[nm];
+			h  = rv1[k];
+			f  = ( (y-z) * (y+z) + (g-h) * (g+h) ) / ( 2.0 * h * y );
+			
+			absf = fabs(f);
+			g    = ( absf > 1.0 ) ?
+			absf * sqrt ( 1.0 + pow ( 1.0/absf , 2.0 ) ) :
+			sqrt ( 1.0 + pow ( absf , 2.0 ) );
+			
+			f = ( (x-z) * (x+z) +
+				 h * ( ( y / ( f + ( (f >= 0)? fabs(g) : -fabs(g) ) ) ) - h ) ) / x;
+			c = s = 1.0;
+			
+			for ( j = l ; j <= nm ; ++j ) {
+				i = j + 1;
+				g = rv1[i];
+				y = W[i];
+				h = s * g;
+				g = c * g;
+				
+				absf = fabs(f);
+				absh = fabs(h);
+				z    = ( absf > absh ) ?
+				absf * sqrt ( 1.0 + pow ( absh/absf , 2.0 ) ) :
+				( ( absh==0 ) ? 0.0 : absh * sqrt ( 1.0 + pow ( absf/absh , 2.0 ) ) );
+				
+				rv1[j] = z;
+				c      = f / z;
+				s      = h / z;
+				f      = x * c + g * s;
+				g      = g * c - x * s;
+				h      = y * s;
+				y     *= c;
+				for ( jj = 0 ; jj < n ; ++jj )
+				{
+					x = V[jj][j];
+					z = V[jj][i];
+					V[jj][j] = x * c + z * s;
+					V[jj][i] = z * c - x * s;
+				}
+				
+				absf = fabs(f);
+				absh = fabs(h);
+				z    = ( absf > absh ) ?
+				absf * sqrt ( 1.0 + pow ( absh/absf , 2.0 ) ) :
+				( ( absh==0 ) ? 0.0 : absh * sqrt ( 1.0 + pow ( absf/absh , 2.0 ) ) );
+				
+				W[j] = z;
+				
+				if ( z ) 
+				{
+					z = 1.0 / z;
+					c = f * z;
+					s = h * z;
+				}
+				f = c * g + s * y;
+				x = c * y - s * g;
+				for ( jj = 0 ; jj < m ; ++jj )
+				{
+					y = M[jj][j];
+					z = M[jj][i];
+					M[jj][j] = y * c + z * s;
+					M[jj][i] = z * c - y * s;
+				}
+			}
+			rv1[l] = 0.0;
+			rv1[k] = f;
+			W  [k] = x;
+		}
+	}
+	
 	delete [] rv1;
- 	return false;
-      }
-      x  = W[l];
-      nm = k - 1;
-      y  = W[nm];
-      g  = rv1[nm];
-      h  = rv1[k];
-      f  = ( (y-z) * (y+z) + (g-h) * (g+h) ) / ( 2.0 * h * y );
-      
-      absf = fabs(f);
-      g    = ( absf > 1.0 ) ?
-	absf * sqrt ( 1.0 + pow ( 1.0/absf , 2.0 ) ) :
-	sqrt ( 1.0 + pow ( absf , 2.0 ) );
-
-      f = ( (x-z) * (x+z) +
-	    h * ( ( y / ( f + ( (f >= 0)? fabs(g) : -fabs(g) ) ) ) - h ) ) / x;
-      c = s = 1.0;
-
-      for ( j = l ; j <= nm ; ++j ) {
- 	i = j + 1;
- 	g = rv1[i];
- 	y = W[i];
- 	h = s * g;
- 	g = c * g;
-
-	absf = fabs(f);
-	absh = fabs(h);
-	z    = ( absf > absh ) ?
-	  absf * sqrt ( 1.0 + pow ( absh/absf , 2.0 ) ) :
-	  ( ( absh==0 ) ? 0.0 : absh * sqrt ( 1.0 + pow ( absf/absh , 2.0 ) ) );
-
- 	rv1[j] = z;
- 	c      = f / z;
- 	s      = h / z;
- 	f      = x * c + g * s;
- 	g      = g * c - x * s;
- 	h      = y * s;
- 	y     *= c;
- 	for ( jj = 0 ; jj < n ; ++jj ) {
- 	  x = V[jj][j];
- 	  z = V[jj][i];
- 	  V[jj][j] = x * c + z * s;
- 	  V[jj][i] = z * c - x * s;
- 	}
-
-	absf = fabs(f);
-	absh = fabs(h);
-	z    = ( absf > absh ) ?
-	  absf * sqrt ( 1.0 + pow ( absh/absf , 2.0 ) ) :
-	  ( ( absh==0 ) ? 0.0 : absh * sqrt ( 1.0 + pow ( absf/absh , 2.0 ) ) );
-
- 	W[j] = z;
-
- 	if ( z ) {
- 	  z = 1.0 / z;
- 	  c = f * z;
- 	  s = h * z;
- 	}
- 	f = c * g + s * y;
- 	x = c * y - s * g;
- 	for ( jj = 0 ; jj < m ; ++jj ) {
- 	  y = M[jj][j];
- 	  z = M[jj][i];
- 	  M[jj][j] = y * c + z * s;
- 	  M[jj][i] = z * c - y * s;
- 	}
-       }
-       rv1[l] = 0.0;
-       rv1[k] = f;
-       W  [k] = x;
-     }
-  }
-
-  delete [] rv1;
-  return true;
+	return true;
 }
